// -*- C++ -*-
/*!
 * @file  InPortConsumer.h
 * @brief InPortConsumer class
 * @date  $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortConsumer.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef InPortConsumer_h
#define InPortConsumer_h

#include <rtm/NVUtil.h>
#include <rtm/CorbaConsumer.h>

namespace RTC
{
  /*!
   * @if jp
   *
   * @class InPortConsumer
   *
   * @brief InPortConsumer ۃNX
   *
   * ̓|[gRV[}̂߂̒ۃC^[tF[XNX
   * eۃNX́Aȉ̏z֐̎񋟂Ȃ΂ȂȂB
   * - push(): f[^M
   * - clone(): |[g̃Rs[
   * - subscribeInterface(): f[^oʒmւ̓o^
   * - unsubscribeInterface(): f[^oʒm̓o^
   *
   * @since 0.4.0
   *
   * @else
   * @class InPortConsumer
   *
   * @brief InPortConsumer abstract class
   *
   * This is the abstract interface class for the input port Consumer.
   * Concrete classes must implement the following pure virtual functions.
   * - push(): Send data
   * - clone(): Copy ports
   * - subscribeInterface(): Subscribe the data send notification
   * - unsubscribeInterface(): Unsubscribe the data send notification
   *
   * @since 0.4.0
   *
   * @endif
   *
   */
  class InPortConsumer
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @endif
     */
    //InPortConsumer();
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortConsumer(){};
    
    /*!
     * @if jp
     * @brief ڑւ̃f[^M
     *
     * ڑ̃|[gփf[^𑗐M邽߂̏z֐B
     *
     * @else
     * @brief Send data to the destination port
     *
     * Pure virtual function to send data to the destination port.
     *
     * @endif
     */
    virtual void push() = 0;
    
    /*!
     * @if jp
     * @brief Y|[g̃Rs[
     *
     * Y|[g̃Rs[𐶐邽߂̏z֐B
     *
     * @return ꂽ InPortConsumer IuWFNg
     *
     * @else
     * @brief Clone this port
     *
     * Pure virtual function to generate this clone port.
     *
     * @return The clone InPortConsumer object
     *
     * @endif
     */
    virtual InPortConsumer* clone() const = 0;
    
    /*!
     * @if jp
     * @brief f[^oʒm󂯎ւ̓o^
     *
     * w肳ꂽvpeB̓eɊÂāAf[^oʒm̎󂯎ɓo^
     * ߂̏z֐B
     *
     * @param properties o^ɎQƂvpeB
     *
     * @return o^
     *
     * @else
     * @brief Subscribe the data send notification
     *
     * Pure virtual function to subscribe the data send notification
     * based on specified property information.
     *
     * @param properties Properties for reference when subscribing
     *
     * @return Subscription result
     *
     * @endif
     */
    virtual bool subscribeInterface(const SDOPackage::NVList& properties) = 0;
    
    /*!
     * @if jp
     * @brief f[^oʒm󂯎肩̓o^
     *
     * f[^oʒm̎󂯎肩o^邽߂̏z֐B
     *
     * @param properties o^ɎQƂvpeB
     *
     * @else
     * @brief Unsubscribe the data send notification
     *
     * Pure virtual function to unsubscribe the data send notification.
     *
     * @param properties Properties for reference when unsubscribing
     *
     * @endif
     */
    virtual void unsubscribeInterface(const SDOPackage::NVList& properties) = 0;
    
  protected:
    
  private:
    
  };
};     // namespace RTC
#endif // InPortConsumer_h
