// -*- C++ -*-
/*!
 * @file ManagerConfig.h
 * @brief RTC manager configuration
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ManagerConfig.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef ManagerConfig_h
#define ManagerConfig_h

#include <rtm/Properties.h>


namespace RTC
{
  /*!
   * @if jp
   *
   * @class ManagerConfig
   * @brief Manager configuration NX
   *
   * Manager ̃RtBM[VsAR}hC󂯎A
   * (邢͈Ȃ)CX^XB
   * R}hCŎw肳ꂽݒt@CAϐȂǂ Manager 
   * vpeBݒ肷B
   *
   * eݒ̗Dx͈ȉ̂ƂłB
   * <OL>
   * <LI>R}hCIvV "-f"
   * <LI>ϐ "RTC_MANAGER_CONFIG"
   * <LI>ftHgݒt@C "./rtc.conf"
   * <LI>ftHgݒt@C "/etc/rtc.conf"
   * <LI>ftHgݒt@C "/etc/rtc/rtc.conf"
   * <LI>ftHgݒt@C "/usr/local/etc/rtc.conf"
   * <LI>ftHgݒt@C "/usr/local/etc/rtc/rtc.conf"
   * <LI>ߍ݃RtBM[Vl
   *</OL>
   * AR}hCIvV "-d" w肳ꂽꍇ́A
   * (Ƃ -f Őݒt@Cw肵Ă)ߍ݃RtBM[Vl
   * gpB
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class ManagerConfig
   * @brief Manager configuration class
   *
   * Modify Manager's configuration. 
   * This class receives the command line arguments and will be instantiated.
   * Set property information of Manager with the configuration file specified
   * by the command line argument or the environment variable etc.
   *
   * The priorities of each configuration are as follows:
   * <OL>
   * <LI>Command option "-f"
   * <LI>Environment variable "RTC_MANAGER_CONFIG"
   * <LI>Default configuration file "./rtc.conf"
   * <LI>Default configuration file "/etc/rtc.conf"
   * <LI>Default configuration file "/etc/rtc/rtc.conf"
   * <LI>Default configuration file "/usr/local/etc/rtc.conf"
   * <LI>Default configuration file "/usr/local/etc/rtc/rtc.conf"
   * <LI>Embedded configuration value
   *</OL>
   * If the command option "-d" is specified (even if specify configuration file
   * by "-f" option), the embedded configuration values will be used.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class ManagerConfig
  {
  public:
    // The list of default configuration file path.
    /*!
     * @if jp
     * @brief Manager RtBM[ṼftHgEt@CEpX
     * @else
     * @brief The default configuration file path for manager
     * @endif
     */
    static const char* config_file_path[];
    
    // Environment value to specify configuration file
    /*!
     * @if jp
     * @brief ftHgERtBM[Ṽt@CEpXʂ
     *        ϐ
     * @else
     * @brief The environment variable to distinguish the default configuration
     *        file path
     * @endif
     */
    static const char* config_file_env;
    
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B(Ȃ)
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor (Do nothing)
     *
     * @endif
     */
    ManagerConfig();
    
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * ^ꂽɂRtBM[V̏sB
     *
     * @param argc R}hC̐
     * @param argv R}hC
     *
     * @else
     *
     * @brief Constructor
     *
     * Initialize configuration information by given arguments.
     *
     * @param argc Number of command line arguments
     * @param argv The command line arguments
     *
     * @endif
     */
    ManagerConfig(int argc, char** argv);
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~ManagerConfig();
    
    /*!
     * @if jp
     *
     * @brief 
     *
     * R}hCɉďsBR}hCIvV
     * ȉ̂̂gp\łB
     *
     *  - -f file   : RtBM[Vt@Cw肷B
     *  - -l module : [h郂W[w肷B()
     *  - -o options: ̑IvVw肷B()
     *  - -d        : ftHg̃RtBM[VgB()
     *
     * @param argc R}hC̐
     * @param argv R}hC
     *
     * @else
     *
     * @brief Initialization
     *
     * Initialize with command line options. The following command options
     * are available.
     *
     * - -f file   : Specify the configuration file.
     * - -l module : Specify modules to be loaded. (Not implemented)
     * - -o options: Specify other options. (Not implemented)
     * - -d        : Use default static configuration. (Not implemented)
     *
     * @param argc Number of command line arguments
     * @param argv The command line arguments
     *
     * @endif
     */
    void init(int argc, char** argv);
    
    /*!
     * @if jp
     * @brief Configuration  Property ɐݒ肷
     * 
     * Manager Configuration w肳ꂽ Property ɐݒ肷B
     *
     * @param prop Configuration ݒΏ Property
     * 
     * @else
     * @brief Specify the configuration information to the Property
     * 
     * Configure to the properties specified by Manager's configuration
     *
     * @param prop The target properties to configure
     * 
     * @endif
     */
    void configure(Properties& prop);
    
    /*!
     * @if jp
     *
     * @brief RtBM[V擾
     *
     * RtBM[V擾Binit()ĂяoOɌĂԂƁA
     * ÓIɒ`ꂽftHg̃RtBM[VԂB
     * init() ĂяoɌĂԂƁAR}hCAϐ
     * ÂꂽRtBM[VԂB
     * ()
     *
     * @return Manager ̃RtBM[V
     *
     * @else
     *
     * @brief Get the configuration.
     *
     * Get the configuration.
     * When this operation is called before calling init() function,
     * return the default configuration statically defined,
     * When this operation is called after calling init() function,
     * return the initialized configuration according to
     * the command line arguments, the environment variables etc.
     * (Not implemented)
     *
     * @return Manager's configuration information
     *
     * @endif
     */
    Properties getConfig() const;
    
  protected:
    /*!
     * @if jp
     *
     * @brief R}hp[X
     *
     * - -f file   : RtBM[Vt@Cw肷B
     * - -l module : [h郂W[w肷B()
     * - -o options: ̑IvVw肷B()
     * - -d        : ftHg̃RtBM[VgB()
     *
     * @param argc R}hC̐
     * @param argv R}hC
     *
     * @else
     *
     * @brief Parse the command arguments
     *
     *  - -f file   : Specify the configuration file.
     *  - -l module : Specify modules to be loaded. (Not implemented)
     *  - -o options: Other options. (Not implemented)
     *  - -d        : Use default static configuration. (Not implemented)
     *
     * @param argc Number of command line arguments
     * @param argv The command line arguments
     *
     * @endif
     */
    void parseArgs(int argc, char** argv);
    
    /*!
     * @if jp
     *
     * @brief Configuration file ̌
     *
     * Configuration file Aݒ肷B
     *  Configuration file ݒς݂̏ꍇ́At@C̑݊mFsB
     *
     * Configuration file ̗D揇<br>
     * R}hIvVw聄ϐftHgt@CftHgݒ
     *
     * ftHgIvV(-d): ftHgt@CĂ
     *                               ftHgݒg
     *
     * @return Configuration file 
     *
     * @else
     *
     * @brief Find the configuration file
     *
     * Find the configuration file and configure it.
     * Confirm the file existence when the configuration file has 
     * already configured.
     *
     * The priority of the configuration file<br>
     * The command optionthe environment variablethe default file
     * the default configuration
     *
     * Default force option(-d): Ignore any default files and use the default 
     * configuration.
     *
     * @return Configuration file search result
     *
     * @endif
     */
    bool findConfigFile();
    
    /*!
     * @if jp
     *
     * @brief VXeݒ肷
     *
     * VXe擾vpeBɃZbgBݒ肳L[͈ȉ̒ʂB
     *  - manager.os.name    : OS
     *  - manager.os.release : OS[X
     *  - maanger.os.version : OSo[W
     *  - manager.os.arch    : OSA[LeN`
     *  - manager.os.hostname: zXg
     *  - manager.pid        : vZXID
     * 
     * @param prop VXeݒ肵vpeB
     * @else
     * 
     * @brief Set system information
     * 
     * Get the following system info. and set them to Manager's properties.
     *  - manager.os.name    : OS name
     *  - manager.os.release : OS release name
     *  - manager.os.version : OS version
     *  - manager.os.arch    : OS architecture
     *  - manager.os.hostname: Hostname
     *  - manager.pid        : process ID
     *
     * @param prop Properties to set system information
     *
     * @endif
     */
    void setSystemInformation(Properties& prop);
    
    /*!
     * @if jp
     * @brief t@C̑݊mF
     *
     * w肳ꂽt@C݂邩mFB
     *
     * @param filename mFΏۃt@C
     *
     * @return Ώۃt@CmF(݂ꍇtrue)
     *
     * @else
     * @brief Check the file existence
     *
     * Confirm whether the specified file exists
     *
     * @param filename The target confirmation file
     *
     * @return file existance confirmation (True if the file exists.)
     *
     * @endif
     */
    bool fileExist(const std::string& filename);
    
    /*!
     * @if jp
     * @brief Manager RtBM[VEt@C̃pX
     * @else
     * @brief Manager's configuration file path
     * @endif
     */
    std::string m_configFile;
  };
}; // namespace RTC  
#endif // ManagerConfig_h
