// -*- C++ -*-
/*!
 * @file RTObject.h
 * @brief RT component base class
 * @date $Date: 2008-01-14 07:57:18 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RTObject.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef RTObject_h
#define RTObject_h

// CORBA header include
#include "rtm/RTC.h"
#include "rtm/Properties.h"
#include "rtm/idl/RTCSkel.h"
#include "rtm/idl/OpenRTMSkel.h"
#include "rtm/PortBase.h"
//#include "rtm/ObjectManager.h"
#include "rtm/PortAdmin.h"
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/InPort.h>
#include <rtm/OutPort.h>
#include <rtm/ConfigAdmin.h>

// ACE
#include <ace/Task.h>

namespace SDOPackage
{
  class Configuration_impl;
};

namespace RTC
{
  class Manager;
  
  /*!
   * @if jp
   * @brief RTR|[lgNX
   *
   * eRTR|[lg̃x[XƂȂNXB
   * Robotic Technology Component dl lightweightRTComponent̎NXB
   * R|[lg̋@\񋟂 ComponentAction C^[tF[X
   * R|[lg̃CtTCNǗs߂ LightweightRTObject ̎
   * 񋟂B
   * ۂɃ[UR|[lg쐬ꍇɂ́AExecution Semantics ɑΉ
   * eTuNX𗘗pB<BR>
   * (̎ł Periodic Sampled Data Processing ̂݃T|[gĂ邽߁A
   *  dataFlowComponent 𒼐ڌpĂ)
   *
   * @since 0.2.0
   *
   * @else
   * @brief RT-Component class
   *
   * This is a class to be a base of each RT-Component.
   * This is a implementation class of lightweightRTComponent in Robotic
   * Technology Component specification.
   * This provides a implementation of ComponentAction interface that offers
   * the the component's function and the implementation of LightweightRTObject
   * for management of the component's lifecycle.
   * When users actually create the components, they should use each subclass
   * corresponding to Execution Semantics.<BR>
   * (In current implementation, since only Periodic Sampled Data Processing is
   * supported, this class inherits dataFlowComponent directly.)
   *
   * @since 0.2.0
   *
   * @endif
   */
  class RTObject_impl
    : public virtual POA_RTC::DataFlowComponent, 
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param manager }l[WIuWFNg
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     * @param manager Manager object
     *
     * @endif
     */
    RTObject_impl(Manager* manager);
    
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param orb ORB
     * @param poa POA
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     * @param orb ORB
     * @param poa POA
     *
     * @endif
     */
    RTObject_impl(CORBA::ORB_ptr orb, PortableServer::POA_ptr poa);
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * @else
     * 
     * @brief Virtual destructor
     * 
     * @endif
     */
    virtual ~RTObject_impl();
    
  protected:
    //============================================================
    // Overridden functions
    //============================================================
    /*!
     * @if jp
     *
     * @brief pR[obN֐
     * 
     * ComponentAction::on_initialize Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏́A{֐I[o[ChĎ
     * KvB
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to initialize
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_initialize was invoked.<BR>
     * As for actual initialization of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The initialize action (on CREATED->ALIVE transition)
    // formaer rtc_init_entry() 
    virtual ReturnCode_t onInitialize();
    
    /*!
     * @if jp
     *
     * @brief IpR[obN֐
     * 
     * ComponentAction::on_finalize Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏ÍA{֐I[o[ChĎ
     * KvB
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to finalize
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_finalize was invoked.<BR>
     * As for actual finalization of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The finalize action (on ALIVE->END transition)
    // formaer rtc_exiting_entry()
    virtual ReturnCode_t onFinalize();
    
    /*!
     * @if jp
     *
     * @brief JnpR[obN֐
     * 
     * ComponentAction::on_startup Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̊JńA{֐I[o[ChĎ
     * KvB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function for startup action
     * 
     * Callback function that is executed when
     * ComponentAction::on_startup was invoked.<BR>
     * As for actual startup of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The startup action when ExecutionContext startup
    // former rtc_starting_entry()
    virtual ReturnCode_t onStartup(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief ~pR[obN֐
     * 
     * ComponentAction::on_shutdown Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̒~́A{֐I[o[ChĎ
     * KvB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function for shutdown action
     * 
     * Callback function that is executed when
     * ComponentAction::on_shutdown was invoked.<BR>
     * As for actual shutdown of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The shutdown action when ExecutionContext stop
    // former rtc_stopping_entry()
    virtual ReturnCode_t onShutdown(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief pR[obN֐
     * 
     * ComponentAction::on_activated Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̊́A{֐I[o[ChĎ
     * KvB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to activate
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_activated was invoked.<BR>
     * As for actual activation of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The activated action (Active state entry action)
    // former rtc_active_entry()
    virtual ReturnCode_t onActivated(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief 񊈐pR[obN֐
     * 
     * ComponentAction::on_deactivated Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̔񊈐́A{֐I[o[ChĎ
     * KvB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to deactivate
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_deactivated was invoked.<BR>
     * As for actual deactivation of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The deactivated action (Active state exit action)
    // former rtc_active_exit()
    virtual ReturnCode_t onDeactivated(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief pR[obN֐
     * 
     * DataFlowComponentAction::on_execute Ă΂ꂽۂɎs
     * R[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̎́A{֐I[o[ChĎ
     * KvB<BR>
     * {֐ Periodic Sampled Data Processing ɂ Two-Pass Execution
     * Pڂ̎spXƂĒIɌĂяoB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to execute periodically
     * 
     * This is a callback function that is executed when
     * DataFlowComponentAction::on_execute is invoked.<BR>
     * As for actual periodic execution of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * This function is invoked periodically as the first execution pass of
     * Two-Pass Execution in Periodic Sampled Data Processing.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The execution action that is invoked periodically
    // former rtc_active_do()
    virtual ReturnCode_t onExecute(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief fpR[obN֐
     * 
     * ComponentAction::on_aborting Ă΂ꂽۂɎsR[obN
     * ֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̒f́A{֐I[o[ChĎ
     * KvB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to abort
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_aborting was invoked.<BR>
     * As for actual abortion of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The aborting action when main logic error occurred.
    // former rtc_aborting_entry()
    virtual ReturnCode_t onAborting(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief G[pR[obN֐
     * 
     * ComponentAction::on_error Ă΂ꂽۂɎsR[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̃G[́A{֐I[o[ChĎ
     * KvB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function for error handling
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_error was invoked.<BR>
     * As for actual error handling of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The error action in ERROR state
    // former rtc_error_do()
    virtual ReturnCode_t onError(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief ZbgpR[obN֐
     * 
     * ComponentAction::on_reset Ă΂ꂽۂɎsR[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̃ZbǵA{֐I[o[ChĎ
     * KvB
     * 
     * @param ec_id QĂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to reset
     * 
     * This is a callback function that is executed when
     * ComponentAction::on_reset was invoked.<BR>
     * As for actual reset of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.
     * 
     * @param ec_id ID of the participant ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The reset action that is invoked resetting
    // This is same but different the former rtc_init_entry()
    virtual ReturnCode_t onReset(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief ԕύXpR[obN֐
     * 
     * DataFlowComponentAction::on_state_update Ă΂ꂽۂɎs
     * R[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏ԕύX́A{֐I[o[ChĎ
     * KvB<BR>
     * {֐ Periodic Sampled Data Processing ɂ Two-Pass Execution
     * Qڂ̎spXƂĒIɌĂяoB
     *
     * @param ec_id QĂ ExecutionContext  ID
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to update the state
     * 
     * This is a callback function that is executed when
     * DataFlowComponentAction::on_state_update was invoked.<BR>
     * As for actual updating the state of each component, since this function is
     * dummy-implemented to return RTC::RTC_OK unconditionally, you need to
     * implement this function by overriding it.<BR>
     * This function is invoked periodically as the second execution pass of
     * Two-Pass Execution in Periodic Sampled Data Processing.
     *
     * @param ec_id ID of the participant ExecutionContext
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The state update action that is invoked after onExecute() action
    // no corresponding operation exists in OpenRTm-aist-0.2.0
    virtual ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);
    
    /*!
     * @if jp
     *
     * @brief ύXʒmpR[obN֐
     * 
     * DataFlowComponentAction::on_rate_changed Ă΂ꂽۂɎs
     * R[obN֐B<BR>
     * {֐͖ RTC::RTC_OK Ԃ悤Ƀ_~[Ă̂ŁA
     * eR|[lg̎ۂ̏ԕύX́A{֐I[o[ChĎ
     * KvB<BR>
     * {֐ Periodic Sampled Data Processing ɂ ExecutionContext 
     * sXVꂽۂɌĂяoB
     *
     * @param ec_id QĂ ExecutionContext  ID
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief Callback function to change execution cycle
     * 
     * This is a callback function that is executed when
     * DataFlowComponentAction::on_rate_changed was invoked.<BR>
     * As for actual changing execution cycle of each component, since this 
     * function is dummy-implemented to return RTC::RTC_OK unconditionally,
     * you need to implement this function by overriding it.<BR>
     * This function is invoked when the execution of ExecutionContext 
     * was updated in Periodic Sampled Data Processing.
     *
     * @param ec_id ID of the participant ExecutionContext
     * 
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    // The action that is invoked when execution context's rate is changed
    // no corresponding operation exists in OpenRTm-aist-0.2.0
    virtual ReturnCode_t onRateChanged(RTC::UniqueId ec_id);
    
  public:
    //============================================================
    // RTC::LightweightRTObject
    //============================================================
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC
     *
     * ̃Iy[VĂяǒʂƂāAComponentAction::on_initialize
     * R[obN֐Ă΂B
     * 
     * 
     * - RTC  CreatedԂ̏ꍇݏsB̏Ԃɂꍇɂ
     *   ReturnCode_t::PRECONDITION_NOT_MET ԂĂяo͎sB
     * - ̃Iy[V RTC ̃~hEGAĂ΂邱Ƃz肵ĂA
     *   AvP[VJ҂͒ڂ̃Iy[VĂԂƂ͑z
     *   ĂȂB
     * 
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief [CORBA interface] IInitialize the RTC that realizes this interface.
     *
     * The invocation of this operation shall result in the invocation of the
     * callback ComponentAction::on_initialize.
     *
     * Constraints
     * - An RTC may be initialized only while it is in the Created state. Any
     *   attempt to invoke this operation while in another state shall fail
     *   with ReturnCode_t::PRECONDITION_NOT_MET.
     * - Application developers are not expected to call this operation
     *   directly; it exists for use by the RTC infrastructure.
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    virtual ReturnCode_t initialize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC I
     *
     * ̃Iy[VĂяǒʂƂ ComponentAction::on_finalize()
     * ĂяoB
     *
     * 
     * - RTC  ExecutionContext ɏĂԂ͏IȂB̏ꍇ́A
     *   ܂ŏ ExecutionContextOperations::remove_component ɂĎQ
     *   Ȃ΂ȂȂBȊȌꍇ́ÃIy[VĂяo
     *   Ȃꍇ ReturnCode_t::PRECONDITION_NOT_ME ŎsB
     * - RTC  Created ԂłꍇAI͍sȂB
     *   ̏ꍇÃIy[VĂяo͂Ȃꍇ
     *   ReturnCode_t::PRECONDITION_NOT_MET ŎsB
     * - ̃Iy[VRTC̃~hEGAĂ΂邱Ƃz肵ĂA
     *   AvP[VJ҂͒ڂ̃Iy[VĂԂƂ͑z
     *   ĂȂB
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief [CORBA interface] Finalize the RTC for preparing it for destruction
     * 
     * This invocation of this operation shall result in the invocation of the
     * callback ComponentAction::on_finalize.
     *
     * Constraints
     * - An RTC may not be finalized while it is participating in any execution
     *   context. It must first be removed with 
     *   ExecutionContextOperations::remove_component. Otherwise, this operation
     *   shall fail with ReturnCode_t::PRECONDITION_NOT_MET. 
     * - An RTC may not be finalized while it is in the Created state. Any 
     *   attempt to invoke this operation while in that state shall fail with 
     *   ReturnCode_t::PRECONDITION_NOT_MET.
     * - Application developers are not expected to call this operation directly;
     *  it exists for use by the RTC infrastructure.
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    virtual ReturnCode_t finalize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC I[i[ł ExecutionContext 
     *        ~ÃRecƋɏI
     *
     *  RTC I[i[ł邷ׂĂ̎sReLXg~B
     *  RTC ̎sReLXgL RTC ɑsReLXg
     * (i.e. sReLXgL RTC ͂Ȃ킿̎sReLXg
     * I[i[łB)ɎQĂꍇAY RTC ͂̃ReLXg
     * Ŕ񊈐Ȃ΂ȂȂB
     * RTC ŝǂ ExecutionContext ł Active Ԃł͂ȂȂA
     *  RTC ƂɊ܂܂ RTC IB
     * 
     * 
     * - RTC ĂȂ΁AI邱Ƃ͂łȂB
     *   Created Ԃɂ RTC  exit() ĂяoꍇA
     *   ReturnCode_t::PRECONDITION_NOT_MET ŎsB
     *
     * @return ReturnCode_t ^̃^[R[h
     * 
     * @else
     *
     * @brief [CORBA interface]top the RTC's execution context(s) and finalize
     *        it along with its contents.
     * 
     * Any execution contexts for which the RTC is the owner shall be stopped. 
     * If the RTC participates in any execution contexts belonging to another
     * RTC that contains it, directly or indirectly (i.e. the containing RTC
     * is the owner of the ExecutionContext), it shall be deactivated in those
     * contexts.
     * After the RTC is no longer Active in any Running execution context, it
     * and any RTCs contained transitively within it shall be finalized.
     *
     * Constraints
     * - An RTC cannot be exited if it has not yet been initialized. Any
     *   attempt to exit an RTC that is in the Created state shall fail with
     *   ReturnCode_t::PRECONDITION_NOT_MET.
     *
     * @return The return code of ReturnCode_t type
     * 
     * @endif
     */
    virtual ReturnCode_t exit()
      throw (CORBA::SystemException); 
    
    /*!
     * @if jp
     *
     * @brief [CORBA interface] RTC  Alive Ԃł邩ǂmFB
     *
     * RTC w肵 ExecutionContext ɑ΂ AliveԂł邩ǂmFB
     * RTC ̏Ԃ Active ł邩AInactive ł邩AError ł邩͎s
     * ExecutionContext ɈˑBȂ킿A ExecutionContext ɑ΂Ă
     * Active  ԂłĂA ExecutionContext ɑ΂Ă Inactive Ԃ
     * Ȃꍇ肦B]āÃIy[V͎w肳ꂽ
     * ExecutionContext ɖ₢킹āA RTC ̏Ԃ ActiveAInactiveA
     * Error ̏ꍇɂ Alive ԂƂĕԂB
     *
     * @return Alive ԊmF
     *
     * @else
     *
     * @brief [CORBA interface] Confirm whether RTC is the alive state
     *
     * A component is alive or not regardless of the execution context from
     * which it is observed. However, whether or not it is Active, Inactive,
     * or in Error is dependent on the execution context(s) in which it is
     * running. That is, it may be Active in one context but Inactive in
     * another. Therefore, this operation shall report whether this RTC is
     * either Active, Inactive or in Error; which of those states a component
     * is in with respect to a particular context may be queried from the
     * context itself.
     *
     * @return Result of Alive state confirmation
     *
     * @endif
     */
    virtual CORBA::Boolean is_alive()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContextList擾
     *
     *  RTC L ExecutionContext ̃Xg擾B
     *
     * @return ExecutionContext Xg
     *
     * @else
     * @brief [CORBA interface] Get ExecutionContextList.
     *
     * This operation returns a list of all execution contexts owned by this RTC.
     *
     * @return ExecutionContext List
     *
     * @endif
     */
    virtual ExecutionContextList* get_contexts()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContext擾
     *
     * w肵nh ExecutionContext 擾B
     * nh ExecutionContext ւ̃}bsÓA RTC CX^X
     * ŗLłBnh͂ RTC  attach_context ۂɎ擾łB
     *
     * @param ec_id 擾Ώ ExecutionContext nh
     *
     * @return ExecutionContext
     *
     * @else
     * @brief [CORBA interface] Get ExecutionContext.
     *
     * Obtain a reference to the execution context represented by the given 
     * handle.
     * The mapping from handle to context is specific to a particular RTC 
     * instance. The given handle must have been obtained by a previous call to 
     * attach_context on this RTC.
     *
     * @param ec_id ExecutionContext handle
     *
     * @return ExecutionContext
     *
     * @endif
     */
    virtual ExecutionContext_ptr get_context(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*
      virtual UniqueId 
      set_execution_context_service(const ExecutionContextService_ptr ec);
    */
    
    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContextattach
     *
     * w肵 ExecutionContext ɂ RTC B RTC Ɗ֘A 
     * ExecutionContext ̃nhԂB
     * ̃Iy[V́AExecutionContextOperations::add_component Ă΂ꂽ
     * ۂɌĂяoBԂꂽnh͑̃NCAgŎgp邱Ƃz
     * ĂȂB
     *
     * @param exec_context  ExecutionContext
     *
     * @return ExecutionContext nh
     *
     * @else
     * @brief [CORBA interface] Attach ExecutionContext
     *
     * Inform this RTC that it is participating in the given execution context. 
     * Return a handle that represents the association of this RTC with the 
     * context.
     * This operation is intended to be invoked by 
     * ExecutionContextOperations::add_component. It is not intended for use by 
     * other clients.
     *
     * @param exec_context Participating ExecutionContext
     *
     * @return ExecutionContext Handle
     *
     * @endif
     */
    UniqueId attach_executioncontext(ExecutionContext_ptr exec_context)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     * @brief [CORBA interface] ExecutionContextdetach
     *
     * w肵 ExecutionContext 炱 RTC ̏B
     * ̃Iy[V́AExecutionContextOperations::remove_component Ă
     * ꂽۂɌĂяoBԂꂽnh͑̃NCAgŎgp邱Ƃ
     * z肵ĂȂB
     * 
     * 
     * - w肳ꂽ ExecutionContext  RTC łɏĂȂꍇɂ́A
     *   ReturnCode_t::PRECONDITION_NOT_MET ԂB
     * - w肳ꂽ ExecutionContext ɂđ΂ RTC Active Ԃł
     *   ɂ́A ReturnCode_t::PRECONDITION_NOT_MET ԂB
     *
     * @param ec_id Ώ ExecutionContextnh
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     * @brief [CORBA interface] Detach ExecutionContext.
     *
     * Inform this RTC that it is no longer participating in the given execution 
     * context.
     * This operation is intended to be invoked by 
     * ExecutionContextOperations::remove_component. It is not intended for use 
     * by other clients.
     * Constraints
     * - This operation may not be invoked if this RTC is not already 
     *   participating in the execution context. Such a call shall fail with 
     *   ReturnCode_t::PRECONDITION_NOT_MET.
     * - This operation may not be invoked if this RTC is Active in the indicated
     *   execution context. Otherwise, it shall fail with 
     *   ReturnCode_t::PRECONDITION_NOT_MET.
     *
     * @param ec_id Detaching ExecutionContext Handle
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    ReturnCode_t detach_executioncontext(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    //============================================================
    // RTC::RTObject
    //============================================================
    /*!
     * @if jp
     *
     * @brief [RTObject CORBA interface] R|[lgvt@C擾
     *
     * YR|[lg̃vt@CԂB 
     *
     * @return R|[lgvt@C
     *
     * @else
     *
     * @brief [RTObject CORBA interface] Get RTC's profile
     *
     * This operation returns the ComponentProfile of the RTC.
     *
     * @return ComponentProfile
     *
     * @endif
     */
    virtual ComponentProfile* get_component_profile()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [RTObject CORBA interface] |[g擾
     *
     * YR|[lgۗL|[g̎QƂԂB
     *
     * @return |[gXg
     *
     * @else
     *
     * @brief [RTObject CORBA interface] Get Ports
     *
     * This operation returns the reference of ports held by RTC.
     *
     * @return PortList
     *
     * @endif
     */
    virtual PortList* get_ports()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [RTObject CORBA interface] ExecutionContextAdmin 擾
     *
     * ̃Iy[V͓Y RTC  ExecutionContextɊ֘A
     * ExecutionContextService ̃XgԂB
     *
     * @return ExecutionContextService Xg
     *
     * @else
     *
     * @brief [RTObject CORBA interface] Get ExecutionContextAdmin
     *
     * This operation returns a list containing an ExecutionContextAdmin for
     * every ExecutionContext owned by the RTC.	
     *
     * @return ExecutionContextService List
     *
     * @endif
     */
    virtual ExecutionContextServiceList* get_execution_context_services()
      throw (CORBA::SystemException);
    
    //============================================================
    // RTC::ComponentAction
    //============================================================
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̏
     *
     * RTC AAlive ԂɑJڂB
     * RTC ŗL̏͂ŎsB
     * ̃Iy[VĂяǒʂƂ onInitialize() R[obN֐
     * ĂяoB
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Initialize RTC
     *
     * The RTC has been initialized and entered the Alive state.
     * Any RTC-specific initialization logic should be performed here.
     * As a result of this operation, onInitialize() callback function is called.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_initialize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̏I
     *
     * RTC jB
     * RTC ŗL̏I͂ŎsB
     * ̃Iy[VĂяǒʂƂ onFinalize() R[obN֐
     * ĂяoB
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Finalize RTC
     *
     * The RTC is being destroyed.
     * Any final RTC-specific tear-down logic should be performed here.
     * As a result of this operation, onFinalize() callback function is called.
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_finalize()
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̊Jn
     *
     * RTC  ExecutionContext  Stopped Ԃ Running Ԃ֑J
     * ꍇɌĂяoB
     * ̃Iy[VĂяǒʂƂ onStartup() R[obN֐
     * ĂяoB
     *
     * @param ec_id ԑJڂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Startup RTC
     *
     * The given execution context, in which the RTC is participating, has 
     * transitioned from Stopped to Running.
     * As a result of this operation, onStartup() callback function is called.
     *
     * @param ec_id ID of ExecutionContext that transited to the state
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_startup(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̒~
     *
     * RTC  ExecutionContext  Running Ԃ Stopped Ԃ֑J
     * ꍇɌĂяoB
     * ̃Iy[VĂяǒʂƂ onShutdown() R[obN֐
     * ĂяoB
     *
     * @param ec_id ԑJڂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Shutdown RTC
     *
     * The given execution context, in which the RTC is participating, has 
     * transitioned from Running to Stopped.
     * As a result of this operation, onShutdown() callback function is called.
     *
     * @param ec_id ID of ExecutionContext that transited to the state
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_shutdown(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̊
     *
     *  ExecutionContext  RTC ꂽۂɌĂяoB
     * ̃Iy[VĂяǒʂƂ onActivated() R[obN֐
     * ĂяoB
     *
     * @param ec_id  ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Activate RTC
     *
     * The RTC has been activated in the given execution context.
     * As a result of this operation, onActivated() callback function is called.
     *
     * @param ec_id ID of activation ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_activated(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̔񊈐
     *
     *  ExecutionContext  RTC 񊈐ꂽۂɌĂяoB
     * ̃Iy[VĂяǒʂƂ onDeactivated() R[obN֐
     * ĂяoB
     *
     * @param ec_id 񊈐 ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Deactivate RTC
     *
     * The RTC has been deactivated in the given execution context.
     * As a result of this operation, onDeactivated() callback function
     * is called.
     *
     * @param ec_id ID of deactivation ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_deactivated(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̃G[Ԃւ̑J
     *
     * RTC  ExecutionContext  Active Ԃ Error Ԃ֑Jڂ
     * ꍇɌĂяoB
     * ̃Iy[V RTC  Error ԂɑJڂۂɈxĂяoB
     * ̃Iy[VĂяǒʂƂ onAborting() R[obN֐
     * ĂяoB
     *
     * @param ec_id ԑJڂ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Transition to Error State
     *
     * The RTC is transitioning from the Active state to the Error state in some
     * execution context.
     * This callback is invoked only a single time for time that the RTC 
     * transitions into the Error state from another state. This behavior is in 
     * contrast to that of on_error.
     * As a result of this operation, onAborting() callback function is invoked.
     *
     * @param ec_id ID of ExecutionContext that transited to the state
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_aborting(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̃G[
     *
     * RTC G[ԂɂۂɌĂяoB
     * RTC G[Ԃ̏ꍇɁAΏۂƂȂ ExecutionContext ExecutionKind 
     * ^C~OŌĂяoBႦ΁A
     * - ExecutionKind  PERIODIC ̏ꍇA{Iy[V
     *   DataFlowComponentAction::on_execute  on_state_update ̑ւɁA
     *   ݒ肳ꂽԁAݒ肳ꂽŌĂяoB
     * - ExecutionKind  EVENT_DRIVEN ̏ꍇA{Iy[V
     *   FsmParticipantAction::on_action Ă΂ꂽۂɁAւɌĂяoB
     * ̃Iy[VĂяǒʂƂ onError() R[obN֐Ăяo
     * B
     *
     * @param ec_id Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Error Processing of RTC
     *
     * The RTC remains in the Error state.
     * If the RTC is in the Error state relative to some execution context when
     * it would otherwise be invoked from that context (according to the 
     * contextfs ExecutionKind), this callback shall be invoked instead. 
     * For example,
     * - If the ExecutionKind is PERIODIC, this operation shall be invoked in 
     *   sorted order at the rate of the context instead of 
     *   DataFlowComponentAction::on_execute and on_state_update.
     * - If the ExecutionKind is EVENT_DRIVEN, this operation shall be invoked 
     *   whenever FsmParticipantAction::on_action would otherwise have been 
     *   invoked.
     * As a result of this operation, onError() callback function is invoked.
     *
     * @param ec_id ID of target ExecutionContext
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_error(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [ComponentAction CORBA interface] RTC ̃Zbg
     *
     * Error Ԃɂ RTC ̃JosAInactive ԂɕA
     * ꍇɌĂяoB
     * RTC ̃Joꍇ Inactive ԂɕA邪AȊO
     * ꍇɂ Error Ԃɗ܂B
     * ̃Iy[VĂяǒʂƂ onReset() R[obN֐Ă
     * oB
     *
     * @param ec_id ZbgΏ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [ComponentAction CORBA interface] Resetting RTC
     *
     * The RTC is in the Error state. An attempt is being made to recover it such
     * that it can return to the Inactive state.
     * If the RTC was successfully recovered and can safely return to the
     * Inactive state, this method shall complete with ReturnCode_t::OK. Any
     * other result shall indicate that the RTC should remain in the Error state.
     * As a result of this operation, onReset() callback function is invoked.
     *
     * @param ec_id ID of target ExecutionContext for the reset
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_reset(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    //============================================================
    // RTC::DataFlowComponentAction
    //============================================================
    /*!
     * @if jp
     *
     * @brief [DataFlowComponentAction CORBA interface] RTC ̒폈()
     *
     * ȉ̏ԂێĂꍇɁAݒ肳ꂽŒIɌĂяoB
     * - RTC  Alive ԂłB
     * - w肳ꂽ ExecutionContext  Running ԂłB
     * {Iy[V́ATwo-Pass Execution ̑ŎsB
     * ̃Iy[VĂяǒʂƂ onExecute() R[obN֐Ă
     * oB
     *
     * 
     * - w肳ꂽ ExecutionContext  ExecutionKind ́A PERIODIC łȂ΂
     *   Ȃ
     *
     * @param ec_id 폈Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [DataFlowComponentAction CORBA interface] Primary Periodic 
     *        Operation of RTC
     *
     * This operation will be invoked periodically at the rate of the given
     * execution context as long as the following conditions hold:
     * - The RTC is Active.
     * - The given execution context is Running
     * This callback occurs during the first execution pass.
     * As a result of this operation, onExecute() callback function is invoked.
     *
     * Constraints
     * - The execution context of the given context shall be PERIODIC.
     *
     * @param ec_id ID of target ExecutionContext for Primary Periodic Operation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_execute(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [DataFlowComponentAction CORBA interface] RTC ̒폈()
     *
     * ȉ̏ԂێĂꍇɁAݒ肳ꂽŒIɌĂяoB
     * - RTC  Alive ԂłB
     * - w肳ꂽ ExecutionContext  Running ԂłB
     * {Iy[V́ATwo-Pass Execution ̑ŎsB
     * ̃Iy[VĂяǒʂƂ onStateUpdate() R[obN֐
     * ĂяoB
     *
     * 
     * - w肳ꂽ ExecutionContext  ExecutionKind ́A PERIODIC łȂ΂
     *   Ȃ
     *
     * @param ec_id 폈Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [DataFlowComponentAction CORBA interface] Secondary Periodic 
     *        Operation of RTC
     *
     * This operation will be invoked periodically at the rate of the given
     * execution context as long as the following conditions hold:
     * - The RTC is Active.
     * - The given execution context is Running
     * This callback occurs during the second execution pass.
     * As a result of this operation, onStateUpdate() callback function is
     * invoked.
     *
     * Constraints
     * - The execution context of the given context shall be PERIODIC.
     *
     * @param ec_id ID of target ExecutionContext for 
     *              Secondary Periodic Operation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_state_update(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    /*!
     * @if jp
     *
     * @brief [DataFlowComponentAction CORBA interface] sύXʒm
     *
     * {Iy[V́AExecutionContext ̎sύXꂽƂʒm
     * ۂɌĂяoB
     * ̃Iy[VĂяǒʂƂ onRateChanged() R[obN֐
     * ĂяoB
     *
     * 
     * - w肳ꂽ ExecutionContext  ExecutionKind ́A PERIODIC łȂ΂
     *   Ȃ
     *
     * @param ec_id 폈Ώ ExecutionContext  ID
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [DataFlowComponentAction CORBA interface] Notify rate changed
     *
     * This operation is a notification that the rate of the indicated execution 
     * context has changed.
     * As a result of this operation, onRateChanged() callback function is 
     * called.
     *
     * Constraints
     * - The execution context of the given context shall be PERIODIC.
     *
     * @param ec_id ID of target ExecutionContext for Operation
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t on_rate_changed(UniqueId ec_id)
      throw (CORBA::SystemException);
    
    //============================================================
    // SDOPackage::SdoSystemElement
    //============================================================
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Organization Xg̎擾 
     *
     * SDOSystemElement 0͂ȏ Organization L邱Ƃ
     * oB SDOSystemElement 1ȏ Organization LĂꍇ
     * ɂ́ÃIy[V͏L Organization ̃XgԂB
     * OrganizationLĂȂ΋̃XgԂB
     *
     * @return LĂ Organization Xg
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get Organization list
     *
     * SDOSystemElement can be the owner of zero or more organizations.
     * If the SDOSystemElement owns one or more Organizations, this operation
     * returns the list of Organizations that the SDOSystemElement owns.
     * If it does not own any Organization, it returns empty list.
     *
     * @return Owned Organization List
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::OrganizationList* get_owned_organizations()
      throw (CORBA::SystemException, SDOPackage::NotAvailable);
    
    //============================================================
    // SDOPackage::SDO
    //============================================================
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO ID ̎擾
     *
     * SDO ID ԂIy[VB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     * 
     * @return    \[Xf[^fŒ`Ă SDO  ID
     * 
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get the SDO ID
     *
     * This operation returns id of the SDO.
     * This operation throws SDOException with one of the following types.
     *
     * @return    id of the SDO defined in the resource data model.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual char* get_sdo_id()
      throw (CORBA::SystemException,
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO ^Cv̎擾
     * 
     * SDO Type ԂIy[VB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     *
     * @return    \[Xf[^fŒ`Ă SDO  Type
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO type
     *
     * This operation returns sdoType of the SDO.
     * This operation throws SDOException with one of the following types.
     *
     * @return    Type of the SDO defined in the resource data model.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual char* get_sdo_type()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO DeviceProfile Xg̎擾 
     *
     * SDO  DeviceProfile ԂIy[VB SDO n[hEGAfoCX
     * Ɋ֘AtĂȂꍇɂ́A DeviceProfile ԂB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     *
     * @return    SDO DeviceProfile
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO DeviceProfile list
     *
     * This operation returns the DeviceProfile of the SDO. If the SDO does not
     * represent any hardware device, then a DeviceProfile with empty values
     * are returned.
     * This operation throws SDOException with one of the following types.
     *
     * @return    The DeviceProfile of the SDO.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::DeviceProfile* get_device_profile()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO ServiceProfile ̎擾 
     *
     * SDO LĂ Service  ServiceProfile ԂIy[VB
     * SDO T[rXLĂȂꍇɂ́ÃXgԂB
     * ̃Iy[V͈ȉ̌^̗O𔭐B
     * 
     * @return    SDO 񋟂SĂ Service  ServiceProfileB
     * 
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO ServiceProfile
     * 
     * This operation returns a list of ServiceProfiles that the SDO has.
     * If the SDO does not provide any service, then an empty list is returned.
     * This operation throws SDOException with one of the following types.
     * 
     * @return    List of ServiceProfiles of all the services the SDO is
     *            providing.
     * 
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::ServiceProfileList* get_service_profiles()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] ServiceProfile̎擾 
     *
     *  "id" Ŏw肳ꂽÕT[rX ServiceProfile ԂB
     * 
     * @param     id SDO Service  ServiceProfile Ɋ֘AtꂽʎqB
     * 
     * @return    w肳ꂽ SDO Service  ServiceProfileB
     * 
     * @exception InvalidParameter "id" Ŏw肵 ServiceProfile ݂ȂB
     *                             "id"  nullB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get specified ServiceProfile
     *
     * This operation returns the ServiceProfile that is specified by the
     * argument "id."
     * 
     * @param     id The identifier referring to one of the ServiceProfiles.
     * 
     * @return    The profile of the specified service.
     * 
     * @exception InvalidParameter The ServiceProfile that is specified by 
     *                             the argument 'id' does not exist or if 'id'
     *                             is 'null.'
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::ServiceProfile* get_service_profile(const char* id)
      throw (CORBA::SystemException, 
	     SDOPackage::InvalidParameter, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] w肳ꂽ SDO Service ̎擾
     *
     * ̃Iy[V͈ "id" Ŏw肳ꂽOɂċʂ
     * SDO  Service ւ̃IuWFNgQƂԂB SDO ɂ񋟂
     * Service ͂ꂼӂ̎ʎqɂʂB
     *
     * @param id SDO Service Ɋ֘AtꂽʎqB
     *
     * @return vꂽ SDO Service ւ̎QƁB
     *
     * 
     * @exception InvalidParameter "id" Ŏw肵 ServiceProfile ݂ȂB
     *                             "id"  nullB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get specified SDO Service's reference
     *
     * This operation returns an object implementing an SDO's service that
     * is identified by the identifier specified as an argument. Different
     * services provided by an SDO are distinguished with different
     * identifiers. See OMG SDO specification Section 2.2.8, "ServiceProfile,"
     * on page 2-12 for more details.
     *
     * @param id The identifier referring to one of the SDO Service
     *
     * @return The reference requested to SDO Service.
     *
     * @exception InvalidParameter Argument gidh is null, or if the 
     *                             ServiceProfile that is specified by argument
     *                            gidh does not exist.
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::SDOService_ptr get_sdo_service(const char* id)
      throw (CORBA::SystemException, 
	     SDOPackage::InvalidParameter, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Configuration IuWFNg̎擾 
     *
     * ̃Iy[V Configuration interface ւ̎QƂԂB
     * Configuration interface ͊e SDO Ǘ邽߂̃C^[tF[X
     * ЂƂłB̃C^[tF[X DeviceProfile, ServiceProfile,
     * Organization Œ`ꂽ SDO ̑lݒ肷邽߂ɎgpB
     * Configuration C^[tF[X̏ڍׂɂẮAOMG SDO specification
     *  2.3.5, p.2-24 QƂ̂ƁB
     *
     * @return SDO  Configuration C^[tF[Xւ̎Q
     *
     * @exception InterfaceNotImplemented SDOConfigurationC^[tF[X
     *                                    ȂB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get Configuration object
     *
     * This operation returns an object implementing the Configuration
     * interface. The Configuration interface is one of the interfaces that
     * each SDO maintains. The interface is used to configure the attributes
     * defined in DeviceProfile, ServiceProfile, and Organization.
     * See OMG SDO specification Section 2.3.5, "Configuration Interface,"
     * on page 2-24 for more details about the Configuration interface.
     *
     * @return The Configuration interface of an SDO.
     *
     * @exception InterfaceNotImplemented The target SDO has no Configuration
     *                                    interface.
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOPackage::Configuration_ptr get_configuration()
      throw (CORBA::SystemException, 
	     SDOPackage::InterfaceNotImplemented, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Monitoring IuWFNg̎擾 
     *
     * ̃Iy[V Monitoring interface ւ̎QƂԂB
     * Monitoring interface  SDO ǗC^[tF[ẌłB
     * ̃C^[tF[X SDO ̃vpeBj^O邽߂
     * gpB
     * Monitoring interface ̏ڍׂɂĂ OMG SDO specification 
     * 2.3.7 "Monitoring Interface" p.2-35 QƂ̂ƁB
     *
     * @return SDO  Monitoring interface ւ̎Q
     *
     * @exception InterfaceNotImplemented SDOConfigurationC^[tF[X
     *                                    ȂB
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get Monitoring object
     *
     * This operation returns an object implementing the Monitoring interface.
     * The Monitoring interface is one of the interfaces that each SDO
     * maintains. The interface is used to monitor the properties of an SDO.
     * See OMG SDO specification Section 2.3.7, "Monitoring Interface," on
     * page 2-35 for more details about the Monitoring interface.
     *
     * @return The Monitoring interface of an SDO.
     *
     * @exception InterfaceNotImplemented The target SDO has no Configuration
     *                                    interface.
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOPackage::Monitoring_ptr get_monitoring()
      throw (CORBA::SystemException, 
	     SDOPackage::InterfaceNotImplemented, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] Organization Xg̎擾 
     *
     * SDO 0ȏ Organization (gD)ɏ邱ƂłB  SDO 
     * 1ȏ Organization ɏĂꍇÃIy[V͏
     * Organization ̃XgԂBSDO  ǂ Organization ɂĂȂ
     * ꍇɂ́ÃXgԂB
     *
     * @return SDO  Organization ̃XgB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [SDO interface] Get Organization list
     *
     * An SDO belongs to zero or more organizations. If the SDO belongs to one
     * or more organizations, this operation returns the list of organizations
     * that the SDO belongs to. An empty list is returned if the SDO does not
     * belong to any Organizations.
     *
     * @return The list of Organizations that the SDO belong to.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     * @endif
     */
    virtual SDOPackage::OrganizationList* get_organizations()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO Status Xg̎擾 
     *
     * ̃Iy[V SDO ̃Xe[^X\ NVList ԂB
     *
     * @return SDO ̃Xe[^XB
     *
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InternalError IG[B
     *
     * @else
     *
     * @brief [SDO interface] Get SDO Status list
     *
     * This operation returns an NVlist describing the status of an SDO.
     *
     * @return The actual status of an SDO.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     * @endif
     */
    virtual SDOPackage::NVList* get_status_list()
      throw (CORBA::SystemException, 
	     SDOPackage::NotAvailable, SDOPackage::InternalError);
    
    /*!
     * @if jp
     * 
     * @brief [SDO interface] SDO Status ̎擾 
     *
     * This operation returns the value of the specified status parameter.
     * 
     * @param name SDO ̃Xe[^X`p[^B
     * 
     * @return w肳ꂽp[^̃Xe[^XlB
     * 
     * @exception SDONotExists ^[QbgSDO݂ȂB({ÓACORBAW
     *                         VXeOOBJECT_NOT_EXISTɃ}bsO)
     * @exception NotAvailable SDO݂͑邪ȂB
     * @exception InvalidParameter  "name"  null 邢݂͑ȂB
     * @exception InternalError IG[B
     * @else
     *
     * @brief [SDO interface] Get SDO Status
     *
     * @param name One of the parameters defining the "status" of an SDO.
     *
     * @return The value of the specified status parameter.
     *
     * @exception SDONotExists The target SDO does not exist.(This exception 
     *                         is mapped to CORBA standard system exception
     *                         OBJECT_NOT_EXIST.)
     * @exception NotAvailable The target SDO is reachable but cannot respond.
     * @exception InvalidParameter The parameter defined by "name" is null or
     *                             does not exist.
     * @exception InternalError The target SDO cannot execute the operation
     *                          completely due to some internal error.
     *
     *
     * @endif
     */
    virtual CORBA::Any* get_status(const char* name)
      throw (CORBA::SystemException, 
	     SDOPackage::InvalidParameter, SDOPackage::NotAvailable,
	     SDOPackage::InternalError);
    
    //============================================================
    // Local interfaces
    //============================================================
    /*!
     * @if jp
     *
     * @brief [local interface] CX^X̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽCX^XԂB
     * 
     * @return CX^X
     * 
     * @else
     *
     * @brief [local interface] Get instance name
     * 
     * Return the instance name that has been set in ComponentProfile.
     * 
     * @return Instance name
     * 
     * @endif
     */
    const char* getInstanceName() {return m_profile.instance_name;}
    
    /*!
     * @if jp
     *
     * @brief [local interface] CX^X̐ݒ
     * 
     * ComponentProfile Ɏw肳ꂽCX^Xݒ肷B
     * 
     * @param instance_name CX^X
     * 
     * @else
     *
     * @brief [local interface] Set instance name
     * 
     * Set the instance name specified in ComponentProfile.
     * 
     * @param instance_name Instance name
     * 
     * @endif
     */
    void setInstanceName(const char* instance_name);
    
    /*!
     * @if jp
     *
     * @brief [local interface] ^̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽ^ԂB
     * 
     * @return ^
     * 
     * @else
     *
     * @brief [local interface] Get type name
     * 
     * Get the type name has been set in ComponentProfile.
     * 
     * @return Type name
     * 
     * @endif
     */
    const char* getTypeName() {return m_profile.type_name;}
    
    /*!
     * @if jp
     *
     * @brief [local interface] Description ̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽ Description ԂB
     * 
     * @return Description
     * 
     * @else
     *
     * @brief [local interface] GetDescription
     * 
     * Get the Description has been set in ComponentProfile.
     * 
     * @return Description
     * 
     * @endif
     */
    const char* getDescription() { return m_profile.description;}
    
    /*!
     * @if jp
     *
     * @brief [local interface] o[W̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽo[WԂB
     * 
     * @return o[W
     * 
     * @else
     *
     * @brief [local interface] Get version information
     * 
     * Get the version information that has been set in
     * ComponentProfile.
     * 
     * @return Version information
     * 
     * @endif
     */
    const char* getVersion() {return m_profile.version;}
    
    /*!
     * @if jp
     *
     * @brief [local interface] x_[̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽx_[ԂB
     * 
     * @return x_[
     * 
     * @else
     *
     * @brief [local interface] Get vendor
     * 
     * Get the vendor information that has been set in ComponentProfile.
     * 
     * @return Vendor information
     * 
     * @endif
     */
    const char* getVendor() {return m_profile.vendor;}
    
    /*!
     * @if jp
     *
     * @brief [local interface] JeS̎擾
     * 
     * ComponentProfile ɐݒ肳ꂽJeSԂB
     * 
     * @return JeS
     * 
     * @else
     *
     * @brief [local interface] Get category information
     * 
     * Get the category information that has been set in ComponentProfile.
     * 
     * @return Category information
     * 
     * @endif
     */
    const char* getCategory() {return m_profile.category;}
    
    /*!
     * @if jp
     *
     * @brief [local interface] Naming Server ̎擾
     * 
     * ݒ肳ꂽ Naming Server ԂB
     * 
     * @return Naming Server Xg
     * 
     * @else
     *
     * @brief [local interface] Get Naming Server information
     * 
     * Get Naming Server information that has been set.
     * 
     * @return Naming Server list
     * 
     * @endif
     */
    std::vector<std::string> getNamingNames();
    
    /*!
     * @if jp
     *
     * @brief [local interface] IuWFNgt@X̐ݒ
     * 
     * RTC  CORBA IuWFNgt@Xݒ肷B
     * 
     * @param rtobj IuWFNgt@X
     * 
     * @else
     *
     * @brief [local interface] Set the object reference
     * 
     * Set RTC's CORBA object reference.
     * 
     * @param rtobj The object reference
     * 
     * @endif
     */
    void setObjRef(const RTObject_ptr rtobj);
    
    /*!
     * @if jp
     *
     * @brief [local interface] IuWFNgt@X̎擾
     * 
     * ݒ肳ꂽ CORBA IuWFNgt@X擾B
     * 
     * @return IuWFNgt@X
     * 
     * @else
     *
     * @brief [local interface] Get the object reference
     * 
     * Get CORBA object reference that has been set
     * 
     * @return The object reference
     *
     * @endif
     */
    RTObject_ptr getObjRef() const;
    
    /*!
     * @if jp
     * 
     * @brief [local interface] RTC ̃vpeBݒ肷
     *
     * RTC ێׂvpeBݒ肷B^vpeB́A
     * ComponentProfile ɐݒ肳ׂȂ΂ȂȂB
     * ̃Iy[V͒ʏ RTC ۂ Manager 
     * Ă΂邱ƂӐ}ĂB
     * 
     * @param prop RTC ̃vpeB
     *
     * @else
     *
     * @brief [local interface] Set RTC property
     *
     * This operation sets the properties to the RTC. The given property
     * values should include information for ComponentProfile.
     * Generally, this operation is designed to be called from Manager, when
     * RTC is initialized
     *
     * @param prop Property for RTC.
     *
     * @endif
     */
    void setProperties(const Properties& prop);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] RTC ̃vpeB擾
     *
     * RTC ێĂvpeBԂB
     * RTCvpeBȂꍇ͋̃vpeBԂB
     * 
     * @return RTC ̃vpeB
     *
     * @else
     *
     * @brief [local interface] Get RTC property
     *
     * This operation returns the properties of the RTC.
     * Empty property would be returned, if RTC has no property.
     *
     * @return Property for RTC.
     *
     * @endif
     */
    Properties& getProperties();
    
    /*!
     * @if jp
     *
     * @brief RtBM[Vp[^̐ݒ
     * 
     * RtBM[Vp[^ƕϐoCh
     * \<VarType\>ƂăRtBM[Vp[^̃f[^^w肷B
     *
     * @param param_name RtBM[Vp[^
     * @param var RtBM[Vp[^i[pϐ
     * @param def_val RtBM[Vp[^ftHgl
     * @param trans RtBM[Vp[^ϊp֐
     *
     * @return ݒ茋(ݒ萬:trueCݒ莸s:false)
     * 
     * @else
     *
     * @brief Setup for configuration parameters
     * 
     * Bind configuration parameter to its variable.
     * Specify the data type of configuration parameter as \<VarType\>.
     *
     * @param param_name Configuration parameter name
     * @param var Variables to store configuration parameter
     * @param def_val Default value of configuration parameter
     * @param trans Function to transform configuration parameter type into 
     *        string format
     *
     * @return Setup result (Successful:true, Failed:false)
     * 
     * @endif
     */
    template <typename VarType>
    bool bindParameter(const char* param_name, VarType& var,
		       const char* def_val,
		       bool (*trans)(VarType&, const char*) = ::stringTo)
    {
      m_configsets.bindParameter(param_name, var, def_val, trans);
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief RtBM[Vp[^̍XV(IDw)
     * 
     * w肵ID̃RtBM[VZbgɐݒ肵lŁA
     * RtBM[Vp[^̒lXV
     *
     * @param config_set ݒΏۂ̃RtBM[VZbgID
     * 
     * @else
     *
     * @brief Update configuration parameters (by ID)
     * 
     * Update configuration parameter value by the value that 
     * set to a configuration set of specified ID.
     *
     * @param config_set The target configuration set's ID for setup
     * 
     * @endif
     */
    void updateParameters(const char* config_set);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] Port o^
     *
     * RTC ێPorto^B
     * Port OANZX\ɂ邽߂ɂ́ÃIy[Vɂ
     * o^ĂȂ΂ȂȂBo^ Port ͂ RTC ɂ
     * PortProfile.name ɂʂBāAPort  RTC ɂāA
     * j[N PortProfile.name Ȃ΂ȂȂB
     * o^ꂽ Port ͓œK؂ɃANeBuꂽA̎QƂ
     * IuWFNgQƂXgɕۑB
     * 
     * @param port RTC ɓo^ Port
     *
     * @else
     *
     * @brief [local interface] Register Port
     *
     * This operation registers a Port held by this RTC.
     * In order to enable access to the Port from outside of RTC, the Port
     * must be registered by this operation. The Port that is registered by
     * this operation would be identified by PortProfile.name in the inside of
     * RTC. Therefore, the Port should have unique PortProfile.name in the RTC.
     * The registering Port would be activated properly, and the reference
     * and the object reference would be stored in lists in RTC.
     *
     * @param port Port which is registered to the RTC
     *
     * @endif
     */
    void registerPort(PortBase& port);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] DataInPort o^
     *
     * RTC ێ DataInPort o^B
     * Port ̃vpeBɃf[^|[gł邱("port.dataport")A
     * TCPgp邱("tcp_any")ݒ肷ƂƂɁA DataInPort 
     * CX^X𐶐Ao^B
     * 
     * @param name port 
     * @param inport o^Ώ DataInPort
     *
     * @else
     * 
     * @brief [local interface] Register DataInPort
     *
     * This operation registers DataInPort held by this RTC.
     * Set "port.dataport" and "tcp_any" to property of Port, and
     * create instances of DataInPort and register it.
     * 
     * @param name Port name
     * @param inport DataInPort which is registered to the RTC
     *
     * @endif
     */
    template <class DataType, template <class DataType> class Buffer>
    void registerInPort(const char* name,
			InPort<DataType, Buffer>& inport)
    {
      std::string propkey("port.dataport.");
      propkey += name;
      propkey += ".tcp_any";
      //      std::cout << "key: " << propkey << std::endl;
      //      std::cout << m_properties;
      m_properties[propkey];
      PortBase* port = new DataInPort(name, inport,
				      *m_properties.getNode(propkey));
      registerPort(*port);
    }
    
    /*!
     * @if jp
     * 
     * @brief [local interface] DataOutPort o^
     *
     * RTC ێ DataOutPorto^B
     * Port ̃vpeBɃf[^|[gł邱("port.dataport")A
     * TCPgp邱("tcp_any")ݒ肷ƂƂɁA DataOutPort 
     * CX^X𐶐Ao^B
     * 
     * @param name port 
     * @param outport o^Ώ DataOutPort
     *
     * @else
     * 
     * @brief [local interface] Register DataOutPort
     *
     * This operation registers DataOutPor held by this RTC.
     * Set "port.dataport" and "tcp_any" to property of Port, and then
     * create instances of DataOutPort and register it.
     * 
     * @param name Port name
     * @param outport DataOutPort which is registered to the RTC
     *
     * @endif
     */
    template <class DataType, template <class DataType> class Buffer>
    void registerOutPort(const char* name,
			 OutPort<DataType, Buffer>& outport)
    {
      std::string propkey("port.dataport.");
      propkey += name;
      propkey += ".tcp_any";
      //      std::cout << "key: " << propkey << std::endl;
      //      std::cout << m_properties;
      m_properties[propkey];
      PortBase* port = new DataOutPort(name, outport,
				       *m_properties.getNode(propkey));
      registerPort(*port);
    }
    
    /*!
     * @if jp
     * 
     * @brief [local interface] Port ̓o^폜
     *
     * RTC ێPort̓o^폜B
     * 
     * @param port 폜Ώ Port
     *
     * @else
     *
     * @brief [local interface] Unregister Port
     *
     * This operation unregisters a Port held by this RTC.
     *
     * @param port Port which is unregistered
     *
     * @endif
     */
    void deletePort(PortBase& port);
    
    /*!
     * @if jp
     * 
     * @brief [local interface] Owɂ Port ̓o^폜
     *
     * ̂w肵 RTC ێPort̓o^폜B
     * 
     * @param port_name 폜Ώ Port 
     *
     * @else
     * 
     * @brief [local interface] Delete Port by specifying its name
     *
     * Delete Port which RTC has by specifying its name.
     * 
     * @param port_name Name of Port which is deleted
     *
     * @endif
     */
    void deletePortByName(const char* port_name);
    
    /*!
     * @if jp
     *
     * @brief S Port ̓o^폜
     *
     * RTC ێSĂ Port 폜B
     *
     * @else
     *
     * @brief Unregister All Ports
     *
     * This operation deactivates the all Ports and deletes the all Port's
     * registrations in the RTC
     *
     * @endif
     */
    void finalizePorts();
    
    /*
      template <class DataType>
      void registerInPort(BufferBase<DataType>& buffer)
      {
      CorbaInPort<DataType>* port;
      p = new CorbaInPort<DataType>(buffer);
      Port_var inport;
      inport = new PortInPort();
      inport->setInPortRef(p->_this());
      }
    */
    
  protected:
    /*!
     * @if jp
     *
     * @brief RTC I
     *
     * RTC ̏IsB
     * ێĂS Port ̓o^ƂƂɁAY CORBA IuWFNg
     * 񊈐ARTC IB
     *
     * @else
     *
     * @brief Shutdown RTC
     *
     * This operation ececutes RTC's termination.
     * This unregisters all Ports, deactivates corresponding CORBA objects and 
     * shuts down RTC.
     *
     * @endif
     */
    void shutdown();
    
  protected:
    /*!
     * @if jp
     * @brief }l[WIuWFNg
     * @else
     * @brief Manager object
     * @endif
     */
    Manager* m_pManager;
    
    /*!
     * @if jp
     * @brief ORB ւ̃|C^
     * @else
     * @brief The pointer to the ORB
     * @endif
     */
    CORBA::ORB_var m_pORB;
    
    /*!
     * @if jp
     * @brief POA ւ̃|C^
     * @else
     * @brief The pointer to the POA
     * @endif
     */
    PortableServer::POA_var m_pPOA;
    
    //============================================================
    // SDO ֌W̕ϐ
    //============================================================
    /*!
     * @if jp
     * @brief SDO ێ organization ̃Xg
     * @else
     * @brief SDO owned organization list
     * @endif
     */
    SDOPackage::OrganizationList m_sdoOwnedOrganizations;
    
    /*!
     * @if jp
     * @brief SDOService ̃vt@CXg
     * @else
     * @brief SDOService Profile List
     * @endif
     */
    SDOPackage::ServiceProfileList m_sdoSvcProfiles;
    
    /*!
     * @if jp
     * @brief SDOService ̃vt@CXgidŃT[`邽߂̃t@N^
     * @else
     * @brief Functor to find from SDOService Profile List by id
     * @endif
     */
    struct svc_name
    {
      svc_name (const char* id) : m_id(id) {};
      bool operator()(const SDOPackage::ServiceProfile& prof)
      {
	return m_id == std::string(prof.id);
      }
      std::string m_id;
    };
    
    /*!
     * @if jp
     * @brief SDO Configuration IuWFNgւ̃|C^
     * @else
     * @brief The pointer to the SDO Configuration Object
     * @endif
     */
    SDOPackage::Configuration_impl* m_pSdoConfigImpl;
    
    /*!
     * @if jp
     * @brief SDO Configuration Interface ւ̃|C^
     * @else
     * @brief The pointer to the SDO Configuration Interface
     * @endif
     */
    SDOPackage::Configuration_var  m_pSdoConfig;
    
    /*!
     * @if jp
     * @brief SDO organization
     * @else
     * @brief SDO organization
     * @endif
     */
    SDOPackage::OrganizationList m_sdoOrganizations;
    
    /*!
     * @if jp
     * @brief SDO Status
     * @else
     * @brief SDO Status
     * @endif
     */
    SDOPackage::NVList m_sdoStatus;
    
    //============================================================
    // RTC ֌W̕ϐ
    //============================================================
    /*!
     * @if jp
     * @brief R|[lgvt@C
     * @else
     * @brief ComponentProfile
     * @endif
     */
    ComponentProfile m_profile;
    
    /*!
     * @if jp
     * @brief IuWFNgt@X
     * @else
     * @brief Object reference
     * @endif
     */
    RTObject_ptr m_objref;
    
    /*!
     * @if jp
     * @brief Port ̃IuWFNgt@X̃Xg
     * @else
     * @brief List of Port Object reference
     * @endif
     */
    PortAdmin m_portAdmin;
    
    /*!
     * @if jp
     * @brief ExecutionContextService ̃Xg
     * @else
     * @brief List of ExecutionContextService
     * @endif
     */
    ExecutionContextServiceList m_execContexts;
    
    /*!
     * @if jp
     * @brief Created ԃtO
     * @else
     * @brief Created Status Flag
     * @endif
     */
    bool m_created;
    
    /*!
     * @if jp
     * @brief Alive ԃtO
     * @else
     * @brief Alive Status Flag
     * @endif
     */
    bool m_alive;
    
    /*!
     * @if jp
     * @brief RTC ̃vpeB
     * @else
     * @brief RTC's Property
     * @endif
     */
    Properties m_properties;
    
    /*!
     * @if jp
     * @brief RtBM[VǗIuWFNg
     * @else
     * @brief Configuration Administrator Object
     * @endif
     */
    ConfigAdmin m_configsets;
    
    //------------------------------------------------------------
    // Functor
    //------------------------------------------------------------
    /*!
     * @if jp
     * @brief NVList pt@N^
     * @else
     * @brief Functor to find NVList
     * @endif
     */
    struct nv_name
    {
      nv_name(const char* name) : m_name(name) {};
      bool operator()(const SDOPackage::NameValue& nv)
      {
	return m_name == std::string(nv.name);
      }
      std::string m_name;
    };
    
    /*!
     * @if jp
     * @brief ExecutionContext pt@N^
     * @else
     * @brief Functor to find ExecutionContext
     * @endif
     */
    struct ec_copy
    {
      ec_copy(ExecutionContextList& eclist)
	: m_eclist(eclist)
      {
      }
      void operator()(ExecutionContextService_ptr ecs)
      {
	CORBA_SeqUtil::push_back(m_eclist, ExecutionContext::_duplicate(ecs));
      }
      ExecutionContextList& m_eclist;
    };
    //    ExecutionContextAdminList m_execContextList;
    
    /*!
     * @if jp
     * @brief RTC 񊈐pt@N^
     * @else
     * @brief Functor to deactivate RTC
     * @endif
     */
    struct deactivate_comps
    {
      deactivate_comps(LightweightRTObject_ptr comp)
	: m_comp(comp)
      {
      }
      void operator()(ExecutionContextService_ptr ec)
      {
	ec->deactivate_component(RTC::LightweightRTObject::_duplicate(m_comp));
      }
      LightweightRTObject_var m_comp;
    };
  };
};
#endif // RTObject
