// -*- C++ -*-
/*!
 * @file RingBuffer.h
 * @brief Defautl Buffer class
 * @date $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: RingBuffer.h 706 2008-04-16 00:36:15Z n-ando $
 *
 */

#ifndef RingBuffer_h
#define RingBuffer_h

#include <rtm/RTC.h>

#include <vector>
#include <algorithm>
//#include <ace/Synch.h>
#include <rtm/BufferBase.h>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lg
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  /*!
   * @if jp
   * @class RingBuffer
   * @brief Oobt@NX
   * 
   * w肵̃Oobt@obt@NXB
   * obt@ŜɃf[^i[ꂽꍇAȍ~̃f[^͌Âf[^
   * ㏑B
   * ]āAobt@ɂ͒߂̃obt@̃f[^̂ݕێB
   *
   * )݂̎ł́AԍŌɊi[f[^̂݃obt@ǂݏo\
   *
   * @param DataType obt@Ɋi[f[^^
   *
   * @since 0.4.0
   *
   * @else
   * @class RingBuffer
   * @brief Ring buffer implementation class
   * 
   * This is the buffer implementation class with ring shaped buffer of
   * specified length.
   * If data is stored in the entire buffer, data from now on will be
   * overwritten from old data one by one.
   * Therefore, only the length of latest data is stored in the buffer.
   *
   * Note: In the current implementation, only last stored data can be read
   *       from the buffer.
   *
   * @param DataType Data type to store in the buffer
   *
   * @since 0.4.0
   *
   * @endif
   */
  template <class DataType>
  class RingBuffer
    : public BufferBase<DataType>
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     * 
     * RXgN^
     * w肳ꂽobt@Ńobt@B
     * Aw肳ꂽQ̏ꍇAQŃobt@B
     *
     * @param length obt@
     * 
     * @else
     *
     * @brief Constructor
     * 
     * Constructor.
     * Initialize the buffer by specified buffer length.
     * However, if the specified length is less than two, the buffer should
     * be initialized by two in length.
     *
     * @param length Buffer length
     * 
     * @endif
     */
    RingBuffer(long int length)
      : m_length(length < 2 ? 2 : length),
	m_oldPtr(0),
	m_newPtr(length < 2 ? 1 : length - 1)
    {
      m_buffer.resize(m_length);
    }
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * zfXgN^B
     * 
     * @else
     *
     * @brief Virtual destractor
     * 
     * Virtual destractor
     * 
     * @endif
     */
    virtual ~RingBuffer(){};
    
    /*!
     * @if jp
     *
     * @brief 
     * 
     * obt@̏sB
     * w肳ꂽlobt@ŜɊi[B
     *
     * @param data pf[^
     * 
     * @else
     *
     * @brief Initialize the buffer
     * 
     * Initialize the buffer.
     * Store the specified value to eitire buffer.
     *
     * @param data Data for initialization
     * 
     * @endif
     */
    void init(DataType& data)
    {
      for (long int i = 0; i < m_length; ++i)
	{
	  put(data);
	}
    }
    
    /*!
     * @if jp
     *
     * @brief obt@擾
     * 
     * obt@擾B
     * 
     * @return obt@
     * 
     * @else
     *
     * @brief Get the buffer length
     * 
     * Get the buffer length.
     * 
     * @return Buffer length
     * 
     *
     * @endif
     */
    virtual long int length() const
    {
      return m_length;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@ɏ
     * 
     * ŗ^ꂽf[^obt@ɏށB
     * 
     * @param value ݑΏۃf[^
     *
     * @return f[^݌(true:ݐԂ)
     * 
     * @else
     *
     * @brief Write data into the buffer
     * 
     * Write data which is given argument into the buffer.
     * 
     * @param value Target data for writing
     *
     * @return Writing result (Always true: writing success is returned)
     * 
     * @endif
     */
    virtual bool write(const DataType& value)
    {
      put(value);
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@ǂݏo
     * 
     * obt@Ɋi[ꂽf[^ǂݏoB
     * 
     * @param value ǂݏof[^
     *
     * @return f[^ǂݏo( true:ǂݏo Ԃ)
     * 
     * @else
     *
     * @brief Readout data from the buffer
     * 
     * Readout data stored into the buffer.
     * 
     * @param value Readout data
     *
     * @return Readout result (Always true: readout success is returned)
     * 
     * @endif
     */
    virtual bool read(DataType& value)
    {
      value = get();
      return true;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@tł邩mF
     * 
     * obt@tmFB(falseԂB)
     *
     * @return tmF(false)
     * 
     * @else
     *
     * @brief Check whether the buffer is full.
     * 
     * Check whether the buffer is full (Always return false)
     *
     * @return Full check result (Always false)
     * 
     * @endif
     */
    virtual bool isFull() const
    {
      return false;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@ł邩mF
     * 
     * obt@mFB
     * 
     * )݂̎ł́C݂̃obt@ʒuɊi[ꂽf[^ǂݏoꂽ
     * ǂԂB( true:f[^ǂݏoρCfalse:f[^ǂݏo)
     *
     * @return mF
     * 
     * @else
     *
     * @brief Check whether the buffer is empty
     * 
     * Check whether the buffer is empty.
     * 
     * Note: In the current implementation, return whether the data which was
     *       stored at a current buffer's position was readout.
     *       (true:it has already readout dataCfalse:it does not readout data)
     *
     * @return Empty check result
     * 
     * @endif
     */
    virtual bool isEmpty() const
    {
      return !(this->isNew());
    }
    
    /*!
     * @if jp
     *
     * @brief ŐVf[^mF
     * 
     * ݂̃obt@ʒuɊi[Ăf[^ŐVf[^mFB
     *
     * @return ŐVf[^mF
     *           ( true:ŐVf[^Df[^͂܂ǂݏoĂȂ
     *            false:ߋ̃f[^Df[^͊ɓǂݏoĂ)
     * 
     * @else
     *
     * @brief Check whether the data is newest
     * 
     * Check whether the data stored at a current buffer position is newest.
     *
     * @return Newest data check result
     *         ( true:Newest data. Data has not been readout yet.
     *          false:Past dataDData has already been readout.)
     * 
     * @endif
     */
    bool isNew() const
    {
      return m_buffer[m_newPtr].isNew();
    }
    
  protected:
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^i[
     * 
     * ŗ^ꂽf[^obt@Ɋi[B
     * 
     * )݂̎ł̓f[^i[ƓɁAf[^̓ǂݏoʒu
     * i[f[^ʒuɐݒ肵ĂB̂߁Aɒ߂Ɋi[f[^
     * 擾`ƂȂĂB
     * 
     * @param data i[Ώۃf[^
     * 
     * @else
     *
     * @brief Store data into the buffer
     * 
     * Store data given by argument into the buffer.
     * 
     * Note: In the current implementation, the data position is set the
     *       readout position of data at the same time of storing data
     *       Therefore, the latest stored data is always got.
     * 
     * @param data Target data for the store
     * 
     * @endif
     */
    virtual void put(const DataType& data)
    {
      m_buffer[m_oldPtr].write(data);
      //      ACE_Guard<ACE_Thread_Mutex> guard(m_Mutex);
      m_newPtr = m_oldPtr;
      m_oldPtr = (++m_oldPtr) % m_length;
    }
    
    /*!
     * @if jp
     *
     * @brief obt@f[^擾
     * 
     * obt@Ɋi[ꂽf[^擾B
     *
     * @return 擾f[^
     * 
     * @else
     *
     * @brief Get data from the buffer
     * 
     * Get data stored into the buffer.
     *
     * @return Data got from buffer
     * 
     * @endif
     */
    virtual const DataType& get()
    {
      return m_buffer[m_newPtr].read();
    }
    
    /*!
     * @if jp
     *
     * @brief ɏރobt@ւ̎QƂ擾
     * 
     * ݃obt@ւ̎QƂ擾B
     * 
     * @return ̏ݑΏۃobt@ւ̎Q
     *
     * @else
     *
     * @brief Get the buffer's reference to be written the next
     * 
     * Get the reference to the buffer that will be written.
     * 
     * @return The buffer's reference to be written the next
     *
     * @endif
     */
    virtual DataType& getRef()
    {
      return m_buffer[m_newPtr].data;
    }
    
  private:
    long int m_length;
    long int m_oldPtr;
    long int m_newPtr;
    
    /*!
     * @if jp
     * @class Data
     * @brief obt@z
     * 
     * obt@f[^i[pzNXB
     *
     * @param D obt@Ɋi[f[^̌^
     *
     * @since 0.4.0
     *
     * @else
     * @brief Buffer array
     * 
     * This is an array class for storing buffer's data.
     *
     * @param D Data type to store into the buffer
     *
     * @since 0.4.0
     *
     * @endif
     */
    template <class D>
    class Data
    {
    public:
      Data() : data(), is_new(false){;}
      inline Data& operator=(const D& other)
      {
	this->data = other;
	this->is_new = true;
	return *this;
      }
      inline void write(const D& other)
      {
	this->is_new = true;
	this->data = other;
      }
      inline D& read()
      {
	this->is_new = false;
	return this->data;
      }
      inline bool isNew() const
      {
	return is_new;
      }
      D    data;
      bool is_new;
    };
    
    std::vector<Data<DataType> > m_buffer;
    
  };
}; // namespace RTC

#endif // RingBuffer_h
